import os
from swepy.swepy import swepy
from swepy.nsidcDownloader import nsidcDownloader
from swepy.process import process
from swepy.ease2Transform import ease2Transform
import glob
import pytest
import datetime
import numpy as np
import pandas as pd

def test_ease_nogrid():
    with pytest.raises(ValueError):
        ease = ease2Transform.ease2Transform("From Sea to Summit")

def test_ease_incorrect_res():
    with pytest.raises(ValueError):
        ease = ease2Transform.ease2Transform("EASE2_N3.126km")

def test_ease_incorrect_proj():
    with pytest.raises(ValueError):
        ease = ease2Transform.ease2Transform("EASE3_W25km")

def test_grid_to_geo():
    row = 359.5
    col = 359.5
    n25g = ease2Transform.ease2Transform("EASE2_N25km")
    assert n25g.grid_to_geographic(row,col) == pytest.approx((90.0,0.0), .1)

def test_verbose():
    n25g = ease2Transform.ease2Transform("EASE2_N25km", verbose=True)
    assert isinstance(n25g, ease2Transform.ease2Transform)


import os
from swepy.nsidcDownloader import nsidcDownloader
import glob
import pytest
import datetime
import numpy as np
import pandas as pd

def test_nD_local():
    date = datetime.date(2010,1,1)
    file = {
        "protocol": "http",
        "server": "localhost:8000",
        "datapool": "MEASURES",
        "dataset": "NSIDC-0630",
        "version": "001",
        "projection": "EASE2",
        "resolution": "6.25km",
        "platform":"F17",
        "sensor": "SSMIS",
        'date1': datetime.date(2010, 1, 1),
        'date2': datetime.date(2010, 1, 1),
        "channel": '19H',
        "grid": "N",
        "pass": "M",
        "algorithm": "SIR",
        "input": "CSU",
        "dataversion": "v1.3"
    }
    nD = nsidcDownloader.nsidcDownloader(no_auth = True)
    nD.download_file(**file)
    list1 = glob.glob('*.nc')
    assert 'NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc' in list1


def test_get_auth():
    with pytest.raises(PermissionError):
        nD = nsidcDownloader.nsidcDownloader('room', 'backend')


def test_file_not_found():
    with pytest.raises(FileNotFoundError):
        date = datetime.date(2010,1,1)
        file = {
            "protocol": "http",
            "server": "localhost:8000",
            "datapool": "MEASURES",
            "dataset": "NSIDC-0630",
            "version": "001",
            "projection": "EASE2",
            "resolution": "6.25km",
            "platform":"F17",
            "sensor": "SSMIS",
            'date1': datetime.date(2010, 1, 1),
            'date2': datetime.date(2010, 1, 1),
            "channel": '19H',
            "grid": "K",
            "pass": "M",
            "algorithm": "SIR",
            "input": "CSU",
            "dataversion": "v1.3"
        }
        nD = nsidcDownloader.nsidcDownloader(no_auth = True)
        nD.download_file(**file)


# Unit Testing for SWEpy
import os
from swepy.swepy import swepy
from swepy.nsidcDownloader import nsidcDownloader
import glob
import pytest
import datetime
import numpy as np
from shutil import copy


def set_login_test(self, username = 'test', password = 'test'):
        """
        Monkey patched set_login function 
        """
        self.local_session = True
        self.nD = nsidcDownloader.nsidcDownloader(folder = os.getcwd(), no_auth=True)
        self.nD.username = 'test'
        self.nD.password = 'test'


swepy.set_login = set_login_test

    

def test_check_params_false():
    start = datetime.date(2010,1,1)
    ul = [66,-145]
    lr = [73,-166]
    s1 = swepy(os.getcwd(), start, start, ul)
    assert s1.check_params() == False
    

def test_check_params_true():
    start = datetime.date(2010,1,1)
    ul = [66,-145]
    lr = [76,-166]
    s1 = swepy(os.getcwd(), start, start, ul, lr, username = 'test', password = 'test')
    assert s1.check_params() == True


def test_set_params_bounds():
    start = datetime.date(2010,1,1)
    s1 = swepy(os.getcwd(), start, start, username = 'test', password = 'test')
    s1.set_grid(ul = [66, -145], lr = [73,-166])
    assert s1.check_params() == True


def test_set_params_auth():
    start = datetime.date(2010,1,1)
    s1 = swepy(os.getcwd(), start, start, ul = [66,-145], lr = [73,-166])
    s1.set_login(username = 'test', password = 'test')
    assert s1.check_params() == True


def test_set_params_dates():
    s1 = swepy(os.getcwd(), ul = [66,-145], lr = [73,-166], username = 'test', password = 'test')
    s1.set_dates(start = datetime.date(2010,1,1), end = datetime.date(2010,1,1))
    assert s1.check_params() == True


def test_get_grid_N():
    s1 = swepy(os.getcwd())
    lat1 = 45
    lat2 = 80
    assert s1.get_grid(lat1, lat2) == "N"


def test_get_grid_T():
    s1 = swepy(os.getcwd())
    lat1 = 30
    lat2 = 30
    assert s1.get_grid(lat1,lat2) == "T"


def test_get_grid_S():
    s1 = swepy(os.getcwd())
    lat1 = -50
    lat2 = -80
    assert s1.get_grid(lat1, lat2) == "S"


def test_get_grid_fails():
    s1 = swepy(os.getcwd())
    lat1 = -62
    lat2 = 70
    with pytest.raises(Exception):
        s1.get_grid(lat1,lat2)

def test_get_xy_N():
    ll_ul = [ 62, -140]
    ll_lr = [73, -166]
    s1 = swepy(os.getcwd(), ul = ll_ul, lr = ll_lr)
    list1 = s1.get_xy(ll_ul, ll_lr)
    assert list1 == pytest.approx([-1988822.7284991555, 2370186.631721887,-457544.84080317616,1835112.1237310776],.1)

def test_get_xy_S():
    ll_lr = [-80,9]
    ll_ul = [-69, -16]
    s1 = swepy(os.getcwd(), ul = ll_ul, lr = ll_lr)
    list1 = s1.get_xy(ll_ul, ll_lr)
    assert list1 == pytest.approx([-642633.6942027323, 2241130.027261451, 174488.41818780638, 1101676.5146265049], .1)

def test_get_xy_none():
    ll_lr = None
    ll_ul = None
    s1 = swepy(os.getcwd())
    with pytest.raises(Exception):
        s1.get_xy(ll_ul, ll_lr)


def test_get_file_high():
    s1 = swepy(os.getcwd(), ul = 'N', lr = 'N', username = 'test', password = 'test')
    date = datetime.datetime(2010,1,1)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '6.25km','platform': 'F17','sensor': 'SSMIS',
                    'date1': datetime.datetime(2010, 1, 1),
                    'date2': datetime.datetime(2010, 1, 1),
                    'channel': '19H','grid': 'N','input': 'CSU',
                    'dataversion': 'v1.3',
                    'pass': 'M','algorithm': 'SIR'}


def test_get_file_low():
    s1 = swepy(os.getcwd(), ul = 'N', lr = 'N', username = 'test', password = 'test', high_res=False)
    date = datetime.datetime(2010,1,1)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '25km','platform': 'F17','sensor': 'SSMIS',
                    'date1': datetime.datetime(2010, 1, 1),
                    'date2': datetime.datetime(2010, 1, 1),
                    'channel': '19H','grid': 'N', 'input': 'CSU',
                    'dataversion': 'v1.3',
                    'pass': 'M','algorithm': 'GRD'}


def test_gf_low_var1():
    s1 = swepy(os.getcwd(), ul = 'N', lr = 'N', username = 'test', password = 'test', high_res=False)
    date = datetime.datetime(2003,11,6)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '25km','platform': 'F14','sensor': 'SSMI',
                    'date1': datetime.datetime(2003,11,6),
                    'date2': datetime.datetime(2003,11,5),
                    'channel': '19H','grid': 'N','input': 'CSU','dataversion': 'v1.3',
                    'pass': 'M','algorithm': 'GRD'}

def test_gf_low_var2():
    s1 = swepy(os.getcwd(), ul = 'N', lr = 'N', username = 'test', password = 'test', high_res=False)
    date = datetime.datetime(2006,11,4)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '25km','platform': 'F15','sensor': 'SSMI',
                    'date1': datetime.datetime(2006,11,4),
                    'date2': datetime.datetime(2006,11,3),
                    'channel': '19H','grid': 'N','input': 'CSU','dataversion': 'v1.3',
                    'pass': 'M','algorithm': 'GRD'}

def test_gf_low_TA():
    s1 = swepy(os.getcwd(), ul = 'T', lr = 'T', username = 'test', password = 'test', high_res=False)
    date = datetime.datetime(2006,11,4)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '25km','platform': 'F15','sensor': 'SSMI',
                    'date1': datetime.datetime(2006,11,4),
                    'date2': datetime.datetime(2006,11,3),
                    'channel': '19H','grid': 'T','input': 'CSU','dataversion': 'v1.3',
                    'pass': 'A','algorithm': 'GRD'}

def test_df_smmr():
    s1 = swepy(os.getcwd(), ul = 'N', lr = 'N', username = 'test', password = 'test')
    date = datetime.datetime(1979,1,1)
    file = s1.get_file(date, "19H")
    assert file == {'protocol':'http', 'server':'localhost:8000','datapool':'MEASURES','resolution': '6.25km','platform': 'NIMBUS7','sensor': 'SMMR',
                    'date1': datetime.datetime(1979, 1, 1),
                    'date2': datetime.datetime(1979, 1, 1),
                    'channel': '18H','grid': 'N','input': 'JPL',
                    'dataversion': 'v1.3',
                    'pass': 'M','algorithm': 'SIR'}



def test_safe_subtract1():
    s1 = swepy(os.getcwd())
    tb19 = np.ones((1,152,153))
    tb37 = np.ones((1,154,153))
    tb = s1.safe_subtract(tb19,tb37)
    assert np.shape(tb) == (1,151,152)


def test_safe_subtract2():
        s1 = swepy(os.getcwd())
        tb19 = np.ones((1,154,152))
        tb37 = np.ones((1,152,153))
        tb = s1.safe_subtract(tb19,tb37)
        assert np.shape(tb) == (1,151,151)



def test_scrape_fail():
    date = datetime.date(2010,1,1)
    ul = 'N'
    lr = 'N'
    s1 = swepy(os.getcwd(),start = date, end = date, ul = 'N', lr = 'N', username = 'test', password = 'test')
    list1 = glob.glob("*.nc")
    assert list1 != ['NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010002-37H-M-SIR-CSU-v1.3.nc','NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010002-19H-M-SIR-CSU-v1.3.nc']

def test_scrape():
    date = datetime.date(2010,1,1)
    ul = 'N'
    lr = 'N'
    s1 = swepy(os.getcwd(),start = date, end = date, ul = 'N', lr = 'N', username = 'test', password = 'test')
    s1.scrape()
    list1 = glob.glob("*.nc")
    assert (list1 == ['NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc','NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc'] or 
        list1 == ['NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc','NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc'])

def test_subset():
    if not os.path.exists('sub'):
        os.mkdir('sub')
    date = datetime.date(2010,1,1)
    s1 = swepy(os.getcwd(),ul = [66,-145], lr = [71,-166])
    setattr(s1,'down19list',['NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc'])
    setattr(s1,'down37list',['NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc'])
    path = os.getcwd()+'/'#'/tests/data/MEASURES/NSIDC-0630.001/2010.01.01/'
    out_dir = path + '/sub/'
    s1.subset(in_dir=path,out_dir19=out_dir,out_dir37=out_dir)
    os.chdir(out_dir)
    list1 = glob.glob('*.nc')
    assert os.stat(list1[0]).st_size < 100000

def test_concat():
    date = datetime.date(2010,1,1)
    s1 = swepy(os.getcwd(),ul = [66,-145], lr = [73,-166])
    setattr(s1,'sub19list',['NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc'])
    setattr(s1,'sub37list',['NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc'])
    s1.concatenate()
    list1 = glob.glob("*all*")
    assert 'all_days_19H.nc' in list1


# def test_concat_subFalse():
#     s1 = swepy(os.getcwd(), ul='N',lr='N')
#     copy('NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc', '/wget/NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc')
#     copy('NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc', '/wget/NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc')
#     setattr(s1,'down19list',['NSIDC-0630-EASE2_N6.25km-F17_SSMIS-2010001-19H-M-SIR-CSU-v1.3.nc'])
#     setattr(s1,'down37list',['NSIDC-0630-EASE2_N3.125km-F17_SSMIS-2010001-37H-M-SIR-CSU-v1.3.nc'])
#     s1.concatenate()


def test_get_directories():
    s1 = swepy(os.getcwd())
    assert os.path.exists(os.getcwd()+'/data') == True


def test_clean_dirs():
    s1 = swepy(os.getcwd())
    try:
        s1.clean_dirs()
        list1 = glob.glob("*nc")
        assert list1 == []
    except PermissionError: 
        assert True

def test_final_concat_fail():
     s1 = swepy(os.getcwd())
     res = s1.final_concat()
     assert res == ('all_days_19H.nc', 'all_days_37H.nc')

# def test_scrape_all():
#     date = datetime.datetime(2010,1,1)
#     s1 = swepy(os.getcwd(), start = date, end = date,ul = 'N', lr = 'N', username = 'test', password='test')
#     res = s1.scrape_all()
#     assert res == ['all_days_19H.nc', 'all_days_37H.nc']
    

# def test_scrape_retry():
#     s1 = swepy(os.getcwd(), f)


from swepy.swepy import swepy
from swepy.process import process
import datetime
import os
import pytest
import numpy as np 


@pytest.fixture
def scraped_files():
    date = datetime.date(2010,1,1)
    ul = 'N'
    lr = 'N'
    s1 = swepy(os.getcwd(),start = date, end = date, ul = 'N', lr = 'N', username = 'test', password = 'test')
    files = s1.scrape()
    return (files[0][0], files[1][0])


@pytest.fixture
def arrays(scraped_files):
    tb19, tb37 = process.get_array(scraped_files[0], scraped_files[1])
    return (tb19,tb37)


@pytest.fixture
def swe(arrays):
    tb19 = process.vector_clean(arrays[0])
    tb37 = process.vector_clean(arrays[1])
    swe = swepy.safe_subtract(tb19=tb19,tb37=tb37)
    return swe


def test_get_array(arrays):
    assert type(arrays[0]) == np.ma.core.MaskedArray


def test_vector_clean(arrays):
    cleantb19 = process.vector_clean(arrays[0])
    assert np.isnan(cleantb19).all() == False


def test_pandas_fill(arrays):
    clean = process.pandas_fill(arrays[0][:,1,1])
    assert np.isnan(clean).all() == False


def test_apply_filter_success(arrays):
    tb19 = process.vector_clean(arrays[0])
    tb37 = process.vector_clean(arrays[1])
    swe = swepy.safe_subtract(tb19 = tb19,tb37 = tb37)
    swe = np.concatenate((np.concatenate((np.concatenate((swe, swe), axis=0), swe), axis = 0), swe), axis=0)
    swe = swe[:,1:2,1:2]
    swe = process.apply_filter(swe)
    assert swe.min() == 0


def test_apply_filter_fail(arrays):
    tb19 = process.vector_clean(arrays[0])
    clean19 = process.apply_filter(tb19)
    assert clean19 == ValueError


def test_apply_filter_large():
    tb19 = np.zeros((100,5,5))
    clean19 = process.apply_filter(tb19)
    assert np.shape(clean19) == (100,5,5)


def test_apply_filter_mp():
    tb19 = np.zeros((100,50,50))
    clean19 = process.apply_filter_mphelper(tb19)
    assert np.shape(clean19) == (100,50,50)


def test_apply_filter_mp_fail():
    tb19 = np.zeros((100,1))
    with pytest.raises(Exception):
        process.apply_filter_mphelper(tb19)


# def test_auto_filter(scraped_files):
#     swe = process.auto_filter(scraped_files[0], scraped_files[1])
#     assert swe.min() == 0





    









import numpy as np 
import pandas as pd 
import matplotlib.pyplot as plt 
from netCDF4 import Dataset
from swepy.swepy import swepy 
from swepy.process import process 
from multiprocessing import Pool, Process, cpu_count
import math


class Analysis():

    def __init__(self, start_date, swe):
        """
        Parameters
        ----------
        start_date: datetime
            Day to start building time series off of 
            Infer length from number of days in data
        """
        self.leap_years = [1992, 1996, 2000, 2004, 2008, 2012, 2016]
        self.swe = swe 
        self.start_date = start_date
        self.time = pd.date_range(start_date, periods = np.shape(self.swe)[0], freq = "D")
        #self.melt_df = self.make_df(time)
        self.year_splits = self.create_year_splits()


    def make_df(self, time = None, columns = ['time', 'count']):
        """
        Given a time array, create dateframe with time and count columns
        MELT ONSET 
        """
        if time is None:
            time = self.time
        df = pd.DataFrame(columns=columns)
        df.time = time 
        df['count'].values[:] = 0
        return df


    def create_year_splits(self):
        """
        Take time array from class and create an array of year split indexes
        MELT ONSET (PLUS)
        """
        years = set(self.time.year)
        years = list(years)
        print("Years in time series: {}".format(years))
        tt = self.start_date.timetuple()
        if tt.tm_yday != 1:
            year_splits = [tt.tm_yday]
        else:
            year_splits = [0]
        for i,year in enumerate(years):
            if years[i-1] in self.leap_years:
                year_splits.append(year_splits[-1]+366)
            else: 
                year_splits.append(year_splits[-1]+365)
        if year_splits[-1] > np.shape(self.swe)[0]:
            year_splits[-1] = np.shape(self.swe)[0]
        return year_splits
    

    def count_melt_onset_index(self, swe): # RENAME __COUNT
        """
        Count the date that each pixel reaches zero for first time of season on a given date. 
        Useful for comparison between years of a given region.

        PARAMETERS
        ----------
        swe: np.array
            swe cube (should be clean and relatively smooth)
        """
        melt_df = self.make_df(self.time)
        for i, year in enumerate(self.year_splits[0:len(self.year_splits)-1]):
        # generate melt date boolean matrix
            bool_1 = np.zeros((np.shape(swe)[1], np.shape(swe)[2]), dtype=bool)
            # loop through the days of this year 
            for d in range(self.year_splits[i],self.year_splits[i+1]-1):
                # loop through x and then y of the image 
                for x in range(np.shape(swe)[1]):
                    for y in range(np.shape(swe)[2]):
                        if bool_1[x,y] == False: 
                            if swe[d,x,y] == 0: # less than 5mm is zero (error)
                                melt_df.loc[d,'count'] += 1
                                bool_1[x,y] = True
        return melt_df


    def count_melt_onset_mp(self): #RENAME, get rid of mp 
        """
        Helper function to manage multi-processing pool for counting 
        melt onset date. 

        Parralelize on the 2nd axis (spatially)

        Returns:
        --------
        df: pandas DataFrame 
                Count of melt dates of every pixel in image 
        """
        cpus = cpu_count()
        swe_parts = np.array_split(self.swe, cpus, axis=2)
        with Pool(cpus) as p:
            parts = p.map(self.count_melt_onset_index, swe_parts)
            df = parts[0] # recombine parts 
            for i in range(1,len(parts)):
                df['count'] = df['count'].add(parts[i]['count'])
            return df


    def mask_year_df(self, df, year):
        """
        Mask single year out of pandas dataframe based on "time" column
        """
        mask = df["time"].dt.year == year 
        return df[mask]


    def melt_date_year(self, df):
        """
        Grab the counts of each year and stick them in own key of dictionary
        Makes for easy comparison of melt times each year.
        """
        df_dict = {}
        counts_dict = {}
        for i in set(self.time.year):
            df_dict[i] = self.mask_year_df(df, i)
            counts_dict[i] = df_dict[i]['count'].values
        return counts_dict

    
    def summer_length(self, smooth_cube):
        """
        Function to track summer length for a pixel over each year 
        Generates hash table of key:value = (x,y): summer length in days 
        """
        shape = np.shape(smooth_cube)
        store = {}
        for x in range(np.shape(smooth_cube)[1]):
            for y in range(np.shape(smooth_cube)[2]):
                lengths = {}
                firstofyear = True
                f = 0
                lastofyear = True
                time = smooth_cube[:,x,y].tolist()
                for index,val in enumerate(time,start=1):
                    if (index%365!=0):
                        if (val == 0 and firstofyear):
                            f = index%365
                            firstofyear = False
                        if (val !=0 and firstofyear == False and lastofyear):
                            summerlength = ((index-1)%365)-f
                            lengths[math.floor((index/365) + 1993)] = lengths.get(math.floor((index/365) + 1993),0) + summerlength
                            lastofyear = False
                    else:
                        firstofyear = True
                        lastofyear = True
                store[(x,y)] = lengths
        return store


    def summer_diff(self, summer_dict, smooth_cube = None):
        """
        Find the average change in summer length by pixel and overall

        PARAMETERS:
        ----------
        summer_dict: dict
            dictionary containing the summer lengths of each year by pixel
        smooth_cube: np.array(x,x,x)
            swe cube of clean swe data, OPTIONAL
        """
        if smooth_cube is None: 
            smooth_cube = self.swe
        shape = np.shape(smooth_cube)
        diffmap = np.zeros((shape[1], shape[2]))
        total_diff = 0 
        for key in summer_dict: # grab new pixel dict
            keys = list(summer_dict[key].keys())
            diff = sum([store1[keys[i]] - store1[keys[i-1]] for i in range(1,len(keys))])
            diffmap[key[0], key[1]] = diff/12
            total_diff += diff / 6
        self.diffmap = diffmap 
        return total_diff/len(summer_dict.keys()), diffmap

    
    def display_summer_change(self, interactive=False):
        """
        Potentially stick in a plotting module???
        """
        im = plt.imshow(self.diffmap)
        plt.colorbar()
        if interactive:
            plt.ion()
            plt.show()
            plt.pause(.001)
            plt.close()
        else:
            plt.show()
        return im 

    
    def display_melt_onset_change(self,dict,year1,year2, interactive=False):
        """
        CURRENTLY ONLY WORKS IF FULL YEARS ARE SCRAPED
        """
        len1 = 366 if year1 in self.leap_years else 365
        len2 = 366 if year2 in self.leap_years else 365
        fig, ax = plt.subplots(1,1,figsize=(15,10))
        try:
            bar1 = plt.bar(np.arange(len1),dict[year1], ec = 'black', width=1, zorder = 2, label = str(year1))
        except KeyError:
            print("Year 1 is not in the time series, please try again")
            raise KeyError
        try:
            bar2 = plt.bar(np.arange(len2), dict[year2], ec = 'black', fc = 'red', width = 1, zorder = 4, alpha = .50, label= str(year2))
        except KeyError:
            print("Year 2 is not in the time series, please try again")
            raise KeyError
        plt.title('Initial Zero SWE Date: {} and {}'.format(str(year1),str(year2)), fontsize = 20)
        plt.xlabel('Day of Year', fontsize = 16)
        plt.ylabel('Count of Pixels to Reach Zero', fontsize = 16)
        plt.grid(.25, zorder = 1)
        plt.xlim([50,200])
        plt.legend()
        if interactive:
            plt.ion()
            plt.show()
            plt.pause(.001)
            plt.close()
        else:
            plt.show()
        return fig


    
    # def summer_length_helper(self):
    #     cpus = cpu_count()
    #     swe_parts = np.array_split(self.swe, cpus, axis=2)
    #     with Pool(cpus) as p:
    #         parts = p.map(self.summer_length, swe_parts)
    #         return parts[0]
    # it doesn't really make sense to parralelize this functionality because it will break the hashing functionality 

"""
Need to decide on a flow here. Do i want the user to have to pick and choose the analysis they want, or do they just instantiate the class and it all happens? 

Only reason not to do it all is each piece is slow and there is a lot of computation to each piece
"""
        

